@@ -2,7 +2,7 @@
 #
 # Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
 # Copyright (c) 2008-2015 California Institute of Technology.
-# Copyright (c) 2016-2023 The Uncertainty Quantification Foundation.
+# Copyright (c) 2016-2024 The Uncertainty Quantification Foundation.
 # License: 3-clause BSD.  The full license text is available at:
 #  - https://github.com/uqfoundation/dill/blob/master/LICENSE
 """
@@ -158,15 +158,17 @@ def get_file_type(*args, **kwargs):
     f.close()
     return t
 
+IS_PYODIDE = sys.platform == 'emscripten'
+
 FileType = get_file_type('rb', buffering=0)
 TextWrapperType = get_file_type('r', buffering=-1)
-BufferedRandomType = get_file_type('r+b', buffering=-1)
+BufferedRandomType = None if IS_PYODIDE else get_file_type('r+b', buffering=-1)
 BufferedReaderType = get_file_type('rb', buffering=-1)
 BufferedWriterType = get_file_type('wb', buffering=-1)
 try:
     from _pyio import open as _open
     PyTextWrapperType = get_file_type('r', buffering=-1, open=_open)
-    PyBufferedRandomType = get_file_type('r+b', buffering=-1, open=_open)
+    PyBufferedRandomType = None if IS_PYODIDE else get_file_type('r+b', buffering=-1, open=_open)
     PyBufferedReaderType = get_file_type('rb', buffering=-1, open=_open)
     PyBufferedWriterType = get_file_type('wb', buffering=-1, open=_open)
 except ImportError:
@@ -1023,9 +1025,12 @@ def _import_module(import_name, safe=False):
             items = import_name.split('.')
             module = '.'.join(items[:-1])
             obj = items[-1]
+            submodule = getattr(__import__(module, None, None, [obj]), obj)
+            if isinstance(submodule, (ModuleType, type)):
+                return submodule
+            return __import__(import_name, None, None, [obj])
         else:
             return __import__(import_name)
-        return getattr(__import__(module, None, None, [obj]), obj)
     except (ImportError, AttributeError, KeyError):
         if safe:
             return None
@@ -1353,7 +1358,6 @@ def _save_file(pickler, obj, open_):
 
 
 @register(FileType) #XXX: in 3.x has buffer=0, needs different _create?
-@register(BufferedRandomType)
 @register(BufferedReaderType)
 @register(BufferedWriterType)
 @register(TextWrapperType)
@@ -1363,8 +1367,15 @@ def save_file(pickler, obj):
     logger.trace(pickler, "# Fi")
     return f
 
+if BufferedRandomType:
+    @register(BufferedRandomType)
+    def save_file(pickler, obj):
+        logger.trace(pickler, "Fi: %s", obj)
+        f = _save_file(pickler, obj, open)
+        logger.trace(pickler, "# Fi")
+        return f
+
 if PyTextWrapperType:
-    @register(PyBufferedRandomType)
     @register(PyBufferedReaderType)
     @register(PyBufferedWriterType)
     @register(PyTextWrapperType)
@@ -1374,6 +1385,15 @@ if PyTextWrapperType:
         logger.trace(pickler, "# Fi")
         return f
 
+    if PyBufferedRandomType:
+        @register(PyBufferedRandomType)
+        def save_file(pickler, obj):
+            logger.trace(pickler, "Fi: %s", obj)
+            f = _save_file(pickler, obj, _open)
+            logger.trace(pickler, "# Fi")
+            return f
+
+
 # The following two functions are based on 'saveCStringIoInput'
 # and 'saveCStringIoOutput' from spickle
 # Copyright (c) 2011 by science+computing ag
@@ -1693,7 +1713,7 @@ def _get_typedict_type(cls, clsdict, attrs, postproc_list):
     for name, value in dict.items(clsdict):
         try:
             base_value = inherited_dict[name]
-            if value is base_value:
+            if value is base_value and hasattr(value, '__qualname__'):
                 to_remove.append(name)
         except KeyError:
             pass
@@ -2018,9 +2038,11 @@ if HAS_CTYPES and hasattr(ctypes, 'pythonapi'):
     _PyCapsule_SetName.argtypes = (ctypes.py_object, ctypes.c_char_p)
     _PyCapsule_SetPointer = ctypes.pythonapi.PyCapsule_SetPointer
     _PyCapsule_SetPointer.argtypes = (ctypes.py_object, ctypes.c_void_p)
+    #from _socket import CAPI as _testcapsule
+    _testcapsule_name = b'dill._dill._testcapsule'
     _testcapsule = _PyCapsule_New(
         ctypes.cast(_PyCapsule_New, ctypes.c_void_p),
-        ctypes.create_string_buffer(b'dill._dill._testcapsule'),
+        ctypes.c_char_p(_testcapsule_name),
         None
     )
     PyCapsuleType = type(_testcapsule)
