@@ -2,7 +2,7 @@
 #
 # Author: Mike McKerns (mmckerns @caltech and @uqfoundation)
 # Copyright (c) 2008-2016 California Institute of Technology.
-# Copyright (c) 2016-2023 The Uncertainty Quantification Foundation.
+# Copyright (c) 2016-2024 The Uncertainty Quantification Foundation.
 # License: 3-clause BSD.  The full license text is available at:
 #  - https://github.com/uqfoundation/dill/blob/master/LICENSE
 """
@@ -150,7 +150,7 @@ def nestedglobals(func, recurse=True):
         if '_GLOBAL' in line:
             name = line.split('(')[-1].split(')')[0]
             if CAN_NULL:
-                names.add(name.replace('NULL + ', ''))
+                names.add(name.replace('NULL + ', '').replace(' + NULL', ''))
             else:
                 names.add(name)
     for co in getattr(func, 'co_consts', tuple()):
